import { CanvasRenderer } from "./canvas.ts";
import { Renderer } from "./renderer.ts";

export class Painter implements Renderer {
    private c: OffscreenCanvas;
    private g: OffscreenCanvasRenderingContext2D;
    private r: Renderer;

    private hash: bigint = 0n;

    constructor(width: number, height: number) {
        this.c = new OffscreenCanvas(width, height);
        this.g = this.c.getContext("2d", { alpha: false })!;
        this.r = new CanvasRenderer(
            this.c as unknown as HTMLCanvasElement,
            this.g as unknown as CanvasRenderingContext2D,
        );
        this.rehash();
    }

    private rehash() {
        this.hash = BigInt(Math.floor(Math.random() * Number.MAX_SAFE_INTEGER));
    }

    render(r: Renderer, x: number, y: number) {
        const data = this.g.getImageData(0, 0, this.c.width, this.c.height);
        r.putImageData(data, x, y, this.hash);
    }

    get width(): number {
        return this.r.width;
    }
    get height(): number {
        return this.r.height;
    }
    clear(color: string): void {
        this.rehash();
        this.r.clear(color);
    }
    fillRect(x: number, y: number, w: number, h: number, color: string): void {
        this.rehash();
        this.r.fillRect(x, y, w, h, color);
    }
    fillCirc(x: number, y: number, radius: number, color: string): void {
        this.rehash();
        this.r.fillCirc(x, y, radius, color);
    }
    putImageData(data: ImageData, x: number, y: number, hash?: bigint): void {
        this.rehash();
        this.r.putImageData(data, x, y, hash);
    }
}
